require 'singleton'
require 'haml'
require 'erb'
require 'nokogiri'
require 'active_support/inflector'
require 'maglove/widget'
require 'maglove/engine/registry'
require 'maglove/engine/configuration'
require 'maglove/engine/scope'
require 'maglove/engine/style_builder'

module Maglove
  module Engine
    def self.config
      Configuration.instance
    end

    def self.configure(&block)
      yield Configuration.instance if block_given?
      Configuration.instance
    end

    def self.widget(identifier, options = {}, scope)
      Maglove::Engine::Registry.instance.widgets[identifier].new(scope, options)
    end

    def self.render(haml, variables = {})
      engine = Haml::Engine.new(haml, remove_whitespace: true)
      scope = Scope.new(variables)
      engine.render(scope)
    rescue StandardError => e
      raise RenderError.new(e.message, e)
    end

    def self.register_widget(identifier, klass)
      Registry.instance.register_widget(identifier, klass)
    end

    def self.register_widgets
      path = File.join(File.dirname(__FILE__), "/widget/*.rb")
      Dir[path].each do |file|
        require file
        identifier = File.basename(file).split(".").first
        klass_name = identifier.camelcase
        Maglove::Engine.register_widget(identifier.to_sym, "Maglove::Widget::#{klass_name}".constantize)
        Maglove::Engine.register_widget("legacy_#{identifier}".to_sym, "Maglove::Widget::Legacy#{klass_name}".constantize)
      end
    end

    class RenderError < StandardError
      attr_reader :original_error
      def initialize(message = "Error", original_error = nil)
        super(message)
        @original_error = original_error
      end
    end
  end
end
