module Maglove
  module Engine
    class Scope
      def initialize(variables = {})
        @variables = variables
      end

      def theme
        @variables[:theme]
      end

      def asset_uri
        var(:asset_uri, Maglove::Engine.config.asset_uri)
      end

      def asset(url)
        "#{asset_uri}/themes/#{theme}/#{url}"
      end

      def root_asset(url)
        "#{asset_uri}/#{url}"
      end

      def widget(identifier, options = {}, &block)
        unless Maglove::Engine::Registry.instance.widgets.keys.include?(identifier.to_sym)
          raise "Widget not found: #{identifier}"
        end
        widget = Maglove::Engine.widget(identifier, options, self)
        widget.render(&block)
      end

      def block(identifier, variables = {})
        variables = @variables.merge(variables)
        haml = Maglove::Engine.config.block_resolver.call(identifier, variables)
        Maglove::Engine.render(haml, variables)
      end

      def var(key, default = nil)
        @variables[key.to_sym] || default
      end

      def variable(key, default = nil)
        var(key, default)
      end

      def style_string(options, *args, &block)
        StyleBuilder.new(options, args).process(&block)
      end

      def method_missing(name, *args, &block)
        if name.to_s.end_with?("_widget") and args.length <= 1
          identifier = "legacy_#{name[0..-8]}".to_sym
          widget(identifier, args.first || {}, &block)
        elsif args.length == 0 and @variables.key?(name.to_sym)
          @variables[name.to_sym]
        else
          super
        end
      end

      def respond_to_missing?(name, include_private = false)
        name.to_s.end_with?("_widget") || super
      end

      def drop_container
        haml_tag :div, class: '_typeloft_widget_drop_container'
      end

      def style(*args, &block)
        style = nil
        if args[-1].class.name == 'Hash'
          style_options = args.pop
          style = style_string(style_options, *style_options.keys)
        end
        classes = args.map { |a| "__#{a}" }
        haml_tag :span, class: classes.join(' '), style: style do
          yield if block_given?
        end
      end

      def link(href, target: nil, &block)
        haml_tag :a, href: href, target: target do
          yield if block_given?
        end
      end

      def font(font_face, &block)
        haml_tag :font, face: font_face do
          yield if block_given?
        end
      end
    end
  end
end
