module Maglove
  module Widget
    class Base
      extend Forwardable
      attr_reader :options, :scope
      def_delegators :@scope, :haml_tag, :haml_concat

      def identifier
        "base"
      end

      def defaults
        {}
      end

      def initialize(scope, options = {})
        @options = defaults.merge(options)
        @scope = scope
      end

      def widget_options
        {}
      end

      def style_string(options, *args, &block)
        scope.style_string(options, *args, &block)
      end

      def render(&block)
        yield self if block_given?
      end
    end

    class V2 < Base
      def widget_options
        attributes = { widget: true }
        @options.each do |key, value|
          attributes[key.to_s.dasherize.to_s] = value
        end
        if attributes["padding"]
          attributes["padding"] = attributes["padding"].join(" ")
        else
          attributes["padding"] = "0 0 0 0"
        end
        attributes
      end

      def render(&block)
        haml_tag "#{identifier}-widget", widget_options do
          yield self if block_given?
        end
      end
    end

    class V1 < Base
      def widget_options
        attributes = {
          :class => "_typeloft_widget",
          :"data-widget-identifier" => identifier
        }
        @options.each do |k, v|
          if [:padding, :margin].include?(k)
            [:top, :right, :bottom, :left].each do |dir|
              attributes["data-attribute-#{k}_#{dir}"] = v
            end
          end
          attributes["data-attribute-#{k}"] = v
        end
        attributes
      end

      def render(&block)
        haml_tag :div, widget_options do
          yield self if block_given?
        end
      end

      def drop_container
        haml_tag :div, class: '_typeloft_widget_drop_container'
      end
    end
  end
end
