module Maglove
  module Widget
    class Columns < V2
      def identifier
        "columns"
      end

      def defaults
        {
          layout: "6x6",
          collapse_options: "sm"
        }
      end

      def column(&block)
        yield self if block_given?
      end
    end

    class LegacyColumns < V1
      attr_reader :columns
      attr_reader :column_count
      attr_reader :total_columns

      def initialize(scope, options = {})
        super
        @column_count = 0
        column_array = @options[:columns].to_s.split("x").map(&:to_i)
        if column_array.length == 1
          @total_columns = column_array[0]
          @columns = Array.new(@total_columns) { 12 / @total_columns }
        else
          @total_columns = column_array.length
          @columns = column_array
        end
      end

      def identifier
        "columns"
      end

      def defaults
        {
          columns: "2",
          style: "default",
          margin_bottom: "",
          collapse_options: "sm"
        }
      end

      def render(&block)
        super do
          haml_tag :div, row_options do
            yield self if block_given?
          end
        end
      end

      def column(&block)
        span = columns[column_count]
        raise "ERROR: Row does not allow column at position #{column_count}" unless span
        @column_count += 1
        phone_cols = (options[:collapse_options] == "xs") ? span : "12"
        haml_tag :div, class: "column col-#{phone_cols} col-tablet-#{span} col-#{options[:collapse_options]}-#{span}" do
          yield self if block_given?
          drop_container
        end
      end

      private

      def row_options
        {
          class: "row row-#{options[:style]}",
          style: style_string(options, :margin_bottom)
        }
      end
    end
  end
end
