module Maglove
  module Widget
    class Slider < V2
      def identifier
        "slider"
      end

      def defaults
        {
          slides: 3,
          height: "auto",
          background_color: "#EEEEEE",
          buttons_position: "bottom",
          autoplay: false,
          autoplay_timeout: 3000
        }
      end

      def slide(&block)
        haml_tag :div, class: "slide" do
          yield self if block_given?
        end
      end
    end

    class LegacySlider < V1
      def identifier
        "slider"
      end

      def defaults
        {
          slides: "3",
          height: "auto",
          margin_bottom: "0px",
          background_color: "#EEEEEE",
          buttons_position: "bottom",
          autoplay: "true",
          autoplay_timeout: "3000"
        }
      end

      def render(&block)
        super do
          haml_tag :div, slider_options do
            yield self if block_given?
          end
        end
      end

      def slide(&block)
        haml_tag :div, class: "item" do
          yield self if block_given?
          drop_container
        end
      end

      private

      def slider_options
        data = { class: slider_classes, autoplay: options[:autoplay], autoplay_timeout: options[:autoplay_timeout] }
        { class: "owl-carousel", style: slider_styles, data: data }
      end

      def slider_styles
        style_string options, :margin, :height, :background_color do |sb|
          sb.add(:min_height, "120px")
          sb.add(:overflow, "hidden")
        end
      end

      def slider_classes
        classes = ["owl-carousel", "owl-loaded", "owl-text-select-on", "owl-theme"]
        classes.push("position-#{options[:buttons_position]}") unless options[:buttons_position].empty?
        classes.join(" ")
      end
    end
  end
end
